@@ -10,14 +10,94 @@
 #ifndef XTENSOR_ZARRAY_IMPL_HPP
 #define XTENSOR_ZARRAY_IMPL_HPP
 
-#include "xtensor/xarray.hpp"
-#include "xtensor/xchunked_array.hpp"
-#include "xtensor/xnoalias.hpp"
-#include "xtensor/xscalar.hpp"
-#include "xtensor/xshape.hpp"
+#include <xtl/xplatform.hpp>
+#include <xtl/xhalf_float.hpp>
+#include <xtensor/xarray.hpp>
+#include <xtensor/xchunked_array.hpp>
+#include <xtensor/xnoalias.hpp>
+#include <xtensor/xscalar.hpp>
+#include <xtensor/xshape.hpp>
 
 namespace xt
 {
+    const std::string endianness_string = (xtl::endianness() == xtl::endian::little_endian) ? "<" : ">";
+
+    template <class T>
+    inline void set_data_type(nlohmann::json& metadata)
+    {
+    }
+
+    template <>
+    inline void set_data_type<bool>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = "bool";
+    }
+
+    template <>
+    inline void set_data_type<uint8_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = "u1";
+    }
+
+    template <>
+    inline void set_data_type<int8_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = "i1";
+    }
+
+    template <>
+    inline void set_data_type<int16_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "i2";
+    }
+
+    template <>
+    inline void set_data_type<uint16_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "u2";
+    }
+
+    template <>
+    inline void set_data_type<int32_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "i4";
+    }
+
+    template <>
+    inline void set_data_type<uint32_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "u4";
+    }
+
+    template <>
+    inline void set_data_type<int64_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "i8";
+    }
+
+    template <>
+    inline void set_data_type<uint64_t>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "u8";
+    }
+
+    template <>
+    inline void set_data_type<xtl::half_float>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "f2";
+    }
+
+    template <>
+    inline void set_data_type<float>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "f4";
+    }
+
+    template <>
+    inline void set_data_type<double>(nlohmann::json& metadata)
+    {
+        metadata["data_type"] = endianness_string + "f8";
+    }
 
     /*************************
      * zarray_expression_tag *
@@ -65,11 +145,17 @@ namespace xt
 
         virtual self_type* clone() const = 0;
 
+        virtual const dynamic_shape<std::ptrdiff_t>& get_strides() const = 0;
+        virtual std::size_t get_offset() const = 0;
+        virtual layout_type layout() const = 0;
+
+        virtual const nlohmann::json& get_metadata() const = 0;
+        virtual void set_metadata(const nlohmann::json& metadata) = 0;
         virtual std::size_t dimension() const = 0;
         virtual const shape_type& shape() const = 0;
         virtual void resize(const shape_type& shape) = 0;
         virtual void resize(shape_type&& shape) = 0;
-        virtual void broadcast_shape(shape_type& shape, bool reuse_cache = 0) const = 0;
+        virtual bool broadcast_shape(shape_type& shape, bool reuse_cache = 0) const = 0;
 
         XTL_IMPLEMENT_INDEXABLE_CLASS()
 
@@ -128,11 +214,17 @@ namespace xt
 
         self_type* clone() const override;
 
+        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
+        std::size_t get_offset() const override;
+        layout_type layout() const override;
+
+        const nlohmann::json& get_metadata() const override;
+        void set_metadata(const nlohmann::json& metadata) override;
         std::size_t dimension() const override;
         const shape_type& shape() const override;
         void resize(const shape_type&) override;
         void resize(shape_type&&) override;
-        void broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
+        bool broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
 
     private:
 
@@ -143,6 +235,7 @@ namespace xt
         CTE m_expression;
         mutable xarray<value_type> m_cache;
         mutable bool m_cache_initialized;
+        nlohmann::json m_metadata;
     };
 
     /*******************
@@ -171,11 +264,17 @@ namespace xt
 
         self_type* clone() const override;
 
+        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
+        std::size_t get_offset() const override;
+        layout_type layout() const override;
+
+        const nlohmann::json& get_metadata() const override;
+        void set_metadata(const nlohmann::json& metadata) override;
         std::size_t dimension() const override;
         const shape_type& shape() const override;
         void resize(const shape_type&) override;
         void resize(shape_type&&) override;
-        void broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
+        bool broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
 
     private:
 
@@ -183,6 +282,7 @@ namespace xt
 
         CTE m_expression;
         xarray<value_type> m_array;
+        nlohmann::json m_metadata;
     };
 
     /******************
@@ -209,17 +309,24 @@ namespace xt
 
         self_type* clone() const override;
 
+        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
+        std::size_t get_offset() const override;
+        layout_type layout() const override;
+
+        const nlohmann::json& get_metadata() const override;
+        void set_metadata(const nlohmann::json& metadata) override;
         std::size_t dimension() const override;
         const shape_type& shape() const override;
         void resize(const shape_type&) override;
         void resize(shape_type&&) override;
-        void broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
+        bool broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
 
     private:
 
         zarray_wrapper(const zarray_wrapper&) = default;
 
         CTE m_array;
+        nlohmann::json m_metadata;
     };
 
     /********************
@@ -257,11 +364,17 @@ namespace xt
 
         self_type* clone() const override;
 
+        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
+        std::size_t get_offset() const override;
+        layout_type layout() const override;
+
+        const nlohmann::json& get_metadata() const override;
+        void set_metadata(const nlohmann::json& metadata) override;
         std::size_t dimension() const override;
         const shape_type& shape() const override;
         void resize(const shape_type& shape) override;
         void resize(shape_type&&) override;
-        void broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
+        bool broadcast_shape(shape_type& shape, bool reuse_cache = 0) const override;
 
         const shape_type& chunk_shape() const override;
 
@@ -275,13 +388,17 @@ namespace xt
         shape_type m_chunk_shape;
         mutable xarray<value_type> m_cache;
         mutable bool m_cache_initialized;
+        mutable dynamic_shape<std::ptrdiff_t> m_strides;
+        mutable bool m_strides_initialized;
+
+        nlohmann::json m_metadata;
 
     };
 
     /***********************
      * zexpression_wrapper *
      ***********************/
-        
+
     template <class CTE>
     template <class E>
     inline zexpression_wrapper<CTE>::zexpression_wrapper(E&& e)
@@ -290,6 +407,7 @@ namespace xt
         , m_cache()
         , m_cache_initialized(false)
     {
+        set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -312,6 +430,39 @@ namespace xt
         return new self_type(*this);
     }
 
+    template <class CTE>
+    inline auto zexpression_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    {
+        compute_cache();
+        return m_cache.get_strides();
+    }
+
+    template <class CTE>
+    inline std::size_t zexpression_wrapper<CTE>::get_offset() const
+    {
+        compute_cache();
+        return m_cache.get_offset();
+    }
+
+    template <class CTE>
+    inline layout_type zexpression_wrapper<CTE>::layout() const
+    {
+        compute_cache();
+        return m_cache.layout();
+    }
+
+    template <class CTE>
+    inline auto zexpression_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    {
+        return m_metadata;
+    }
+
+    template <class CTE>
+    inline void zexpression_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    {
+        m_metadata = metadata;
+    }
+
     template <class CTE>
     inline std::size_t zexpression_wrapper<CTE>::dimension() const
     {
@@ -338,11 +489,11 @@ namespace xt
     }
 
     template <class CTE>
-    inline void zexpression_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    inline bool zexpression_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
         return m_expression.broadcast_shape(shape, reuse_cache);
     }
-    
+
     template <class CTE>
     inline void zexpression_wrapper<CTE>::compute_cache() const
     {
@@ -364,6 +515,7 @@ namespace xt
         , m_expression(std::forward<E>(e))
         , m_array(m_expression())
     {
+        set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -383,7 +535,37 @@ namespace xt
     {
         return new self_type(*this);
     }
-    
+
+    template <class CTE>
+    inline auto zscalar_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    {
+        return m_array.strides();
+    }
+
+    template <class CTE>
+    inline std::size_t zscalar_wrapper<CTE>::get_offset() const
+    {
+        return 0;
+    }
+
+    template <class CTE>
+    inline layout_type zscalar_wrapper<CTE>::layout() const
+    {
+        return m_array.layout();
+    }
+
+    template <class CTE>
+    inline auto zscalar_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    {
+        return m_metadata;
+    }
+
+    template <class CTE>
+    inline void zscalar_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    {
+        m_metadata = metadata;
+    }
+
     template <class CTE>
     inline std::size_t zscalar_wrapper<CTE>::dimension() const
     {
@@ -409,27 +591,57 @@ namespace xt
     }
 
     template <class CTE>
-    inline void zscalar_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    inline bool zscalar_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
-        m_array.broadcast_shape(shape, reuse_cache);
+        return m_array.broadcast_shape(shape, reuse_cache);
     }
-    
+
     /******************
      * zarray_wrapper *
      ******************/
 
+    namespace detail
+    {
+        template <class T>
+        struct zarray_wrapper_helper
+        {
+            static inline xarray<T>& get_array(xarray<T>& ar)
+            {
+                return ar;
+            }
+
+            static inline xarray<T>& get_array(const xarray<T>&)
+            {
+                throw std::runtime_error("Cannot return non const array from const array");
+            }
+
+            template <class S>
+            static inline void resize(xarray<T>& ar, S&& shape)
+            {
+                ar.resize(std::forward<S>(shape));
+            }
+
+            template <class S>
+            static inline void resize(const xarray<T>&, S&&)
+            {
+                throw std::runtime_error("Cannot resize const array");
+            }
+        };
+    }
+
     template <class CTE>
     template <class E>
     inline zarray_wrapper<CTE>::zarray_wrapper(E&& e)
         : base_type()
         , m_array(std::forward<E>(e))
     {
+        set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
     inline auto zarray_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
-        return m_array;
+        return detail::zarray_wrapper_helper<value_type>::get_array(m_array);
     }
 
     template <class CTE>
@@ -444,6 +656,36 @@ namespace xt
         return new self_type(*this);
     }
 
+    template <class CTE>
+    inline auto zarray_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    {
+        return m_array.strides();
+    }
+
+    template <class CTE>
+    inline std::size_t zarray_wrapper<CTE>::get_offset() const
+    {
+        return 0;
+    }
+
+    template <class CTE>
+    inline layout_type zarray_wrapper<CTE>::layout() const
+    {
+        return m_array.layout();
+    }
+
+    template <class CTE>
+    inline auto zarray_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    {
+        return m_metadata;
+    }
+
+    template <class CTE>
+    inline void zarray_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    {
+        m_metadata = metadata;
+    }
+
     template <class CTE>
     inline std::size_t zarray_wrapper<CTE>::dimension() const
     {
@@ -459,21 +701,21 @@ namespace xt
     template <class CTE>
     inline void zarray_wrapper<CTE>::resize(const shape_type& shape)
     {
-        m_array.resize(shape);
+        detail::zarray_wrapper_helper<value_type>::resize(m_array, shape);
     }
 
     template <class CTE>
     inline void zarray_wrapper<CTE>::resize(shape_type&& shape)
     {
-        m_array.resize(std::move(shape));
+        detail::zarray_wrapper_helper<value_type>::resize(m_array, std::move(shape));
     }
 
     template <class CTE>
-    inline void zarray_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    inline bool zarray_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
-        m_array.broadcast_shape(shape, reuse_cache);
+        return m_array.broadcast_shape(shape, reuse_cache);
     }
-    
+
     /********************
      * zchunked_wrapper *
      ********************/
@@ -486,10 +728,12 @@ namespace xt
         , m_chunk_shape(m_chunked_array.chunk_shape().size())
         , m_cache()
         , m_cache_initialized(false)
+        , m_strides_initialized(false)
     {
         std::copy(m_chunked_array.chunk_shape().begin(),
                   m_chunked_array.chunk_shape().end(),
                   m_chunk_shape.begin());
+        set_data_type<value_type>(m_metadata);
     }
 
     template <class CTE>
@@ -512,6 +756,41 @@ namespace xt
         return new self_type(*this);
     }
 
+    template <class CTE>
+    inline auto zchunked_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    {
+        if (!m_strides_initialized)
+        {
+            m_strides = detail::get_strides<XTENSOR_DEFAULT_TRAVERSAL>(m_chunked_array);
+            m_strides_initialized = true;
+        }
+        return m_strides;
+    }
+
+    template <class CTE>
+    inline std::size_t zchunked_wrapper<CTE>::get_offset() const
+    {
+        return 0;
+    }
+
+    template <class CTE>
+    inline layout_type zchunked_wrapper<CTE>::layout() const
+    {
+        return m_chunked_array.layout();
+    }
+
+    template <class CTE>
+    inline auto zchunked_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    {
+        return m_metadata;
+    }
+
+    template <class CTE>
+    inline void zchunked_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    {
+        m_metadata = metadata;
+    }
+
     template <class CTE>
     inline std::size_t zchunked_wrapper<CTE>::dimension() const
     {
@@ -537,13 +816,13 @@ namespace xt
         //TODO: check this semantic
         throw std::runtime_error("Cannot resize chunked wrapper");
     }
-    
+
     template <class CTE>
-    inline void zchunked_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    inline bool zchunked_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
-        m_chunked_array.broadcast_shape(shape, reuse_cache);
+        return m_chunked_array.broadcast_shape(shape, reuse_cache);
     }
-    
+
     template <class CTE>
     inline auto zchunked_wrapper<CTE>::chunk_shape() const -> const shape_type&
     {
@@ -581,8 +860,8 @@ namespace xt
         {
         };
 
-        template <class CS, class E>
-        struct is_chunked_array<xchunked_array<CS, E>> : std::true_type
+        template <class CS>
+        struct is_chunked_array<xchunked_array<CS>> : std::true_type
         {
         };
 
